// MEX function to create Convex interpolation constraint matrix
// [A,b]=CvxMat(W,K,p,w0,k0)
// Output: A is sparse, b is dense and should be converted to sparse in MATLAB

// Last Modified: A. d'Aspremont, April 2005
// http://www.carva.org/alexandre.daspremont


#include "CvxMat.h" 

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  double *bvec, *Wmat, *pvec, *Kvec, *w0vec, k0, *sr;
  int i,j,u,n,m,k,nnz,*jcs,*irs;

  Wmat = mxGetPr(prhs[0]);
  m = mxGetM(prhs[0]);
  n = mxGetN(prhs[0]);
  Kvec = mxGetPr(prhs[1]);
  pvec = mxGetPr(prhs[2]);
  w0vec= mxGetPr(prhs[3]);
  k0= mxGetScalar(prhs[4]);


  nnz=(m+1)*(m+1)*(n+1)+(m+1);
  plhs[0] = mxCreateSparse((m+1)*(m+1),(m+1)*(n+1)+1,nnz,mxREAL);
  sr  = mxGetPr(plhs[0]);
  irs = mxGetIr(plhs[0]);
  jcs = mxGetJc(plhs[0]);
  plhs[1] = mxCreateDoubleMatrix((m+1)*(m+1),1,mxREAL); 
  bvec = mxGetPr(plhs[1]);


  // Write matrix
  k=0;
  for (i=0;i<(m+1);i++) 
  {
	  for (j=0;j<n;j++)
	  {
		  jcs[i*(n+1)+j]=k; // Weights
		  for (u=0;u<m;u++)
		  {
			  sr[k]=Wmat[j*m+u];
			  irs[k]=i*(m+1)+u;
			  bvec[i*(m+1)+u]=pvec[u];
			  k++;
		  }
		  sr[k]=w0vec[j];
		  irs[k]=i*(m+1)+m;
		  k++;
	  }
	  jcs[i*(n+1)+n]=k; // Strikes
	  for (u=0;u<m;u++)
	  {
		  sr[k]=Kvec[u];
		  irs[k]=i*(m+1)+u;
		  bvec[i*(m+1)+u]=pvec[u];
		  k++;	  
	  }
	  sr[k]=k0;
	  irs[k]=i*(m+1)+m;
	  k++;
  }
  jcs[(m+1)*(n+1)]=k; // Last column in -1 
  for (i=0;i<(m+1);i++)
  {
	  sr[k]=-1.0;
	  irs[k]=i*(m+1)+m;
	  bvec[i*(m+1)+m]=0.0;
	  k++;
  }
  jcs[(m+1)*(n+1)+1]=k;
}

